@@ -1,6 +1,6 @@
 #' Distance between two locations
 #'
-#' `step_geodist` creates a a *specification* of a
+#' `step_geodist` creates a *specification* of a
 #'  recipe step that will calculate the distance between
 #'  points on a map to a reference location.
 #'
@@ -40,7 +40,7 @@
 #'                ref_lat = 38.8986312, ref_lon = -77.0062457) %>%
 #'   prep(training = Smithsonian)
 #'
-#' juice(near_station) %>%
+#' bake(near_station, new_data = NULL) %>%
 #'   arrange(geo_dist)
 #'
 #' tidy(near_station, number = 1)
@@ -57,13 +57,13 @@ step_geodist <- function(recipe,
                          skip = FALSE,
                          id = rand_id("geodist")) {
   if (length(ref_lon) != 1 || !is.numeric(ref_lon))
-    stop("`ref_lon` should be a single numeric value.", call. = FALSE)
+    rlang::abort("`ref_lon` should be a single numeric value.")
   if (length(ref_lat) != 1 || !is.numeric(ref_lat))
-    stop("`ref_lat` should be a single numeric value.", call. = FALSE)
+    rlang::abort("`ref_lat` should be a single numeric value.")
   if (length(log) != 1 || !is.logical(log))
-    stop("`log` should be a single logical value.", call. = FALSE)
+    rlang::abort("`log` should be a single logical value.")
   if (length(name) != 1 || !is.character(name))
-    stop("`name` should be a single character value.", call. = FALSE)
+    rlang::abort("`name` should be a single character value.")
 
   add_step(
     recipe,
@@ -105,17 +105,19 @@ step_geodist_new <-
 
 #' @export
 prep.step_geodist <- function(x, training, info = NULL, ...) {
-  lon_name <- terms_select(x$lon, info = info)
+  lon_name <- eval_select_recipes(x$lon, training, info)
+  lat_name <- eval_select_recipes(x$lat, training, info)
+
   if (length(lon_name) > 1)
-    stop("`lon` should resolve to a single column name.", call. = FALSE)
+    rlang::abort("`lon` should resolve to a single column name.")
   check_type(training[, lon_name])
-  lat_name <- terms_select(x$lat, info = info)
+
   if (length(lat_name) > 1)
-    stop("`lat` should resolve to a single column name.", call. = FALSE)
+    rlang::abort("`lat` should resolve to a single column name.")
   check_type(training[, lat_name])
 
   if (any(names(training) == x$name))
-    stop("'", x$name, "' is already used in the data.", call. = FALSE)
+    rlang::abort("'", x$name, "' is already used in the data.")
 
   step_geodist_new(
     lon = x$lon,
