@@ -91,7 +91,7 @@ step_hyperbolic_new <-
 
 #' @export
 prep.step_hyperbolic <- function(x, training, info = NULL, ...) {
-  col_names <- terms_select(x$terms, info = info)
+  col_names <- eval_select_recipes(x$terms, training, info)
   check_type(training[, col_names])
 
   step_hyperbolic_new(
