@@ -101,14 +101,14 @@ step_ica <-
            role = "predictor",
            trained = FALSE,
            num_comp  = 5,
-           options = list(),
+           options = list(method = "C"),
            res = NULL,
            prefix = "IC",
            skip = FALSE,
            id = rand_id("ica")) {
 
 
-    recipes_pkg_check(c("dimRed", "fastICA"))
+    recipes_pkg_check(required_pkgs.step_ica())
 
     add_step(
       recipe,
@@ -144,7 +144,7 @@ step_ica_new <-
 
 #' @export
 prep.step_ica <- function(x, training, info = NULL, ...) {
-  col_names <- terms_select(x$terms, info = info)
+  col_names <- eval_select_recipes(x$terms, training, info)
   check_type(training[, col_names])
 
   if (x$num_comp > 0) {
@@ -257,3 +257,10 @@ tunable.step_ica <- function(x, ...) {
     component_id = x$id
   )
 }
+
+
+#' @rdname required_pkgs.step
+#' @export
+required_pkgs.step_ica <- function(x, ...) {
+  c("dimRed", "fastICA")
+}
