@@ -110,7 +110,8 @@ step_log_new <-
 
 #' @export
 prep.step_log <- function(x, training, info = NULL, ...) {
-  col_names <- terms_select(x$terms, info = info)
+  col_names <- eval_select_recipes(x$terms, training, info)
+
   check_type(training[, col_names])
 
   step_log_new(
@@ -139,7 +140,7 @@ bake.step_log <- function(object, new_data, ...) {
         log(new_data[[ col_names[i] ]] + object$offset, base = object$base)
   } else {
     if (object$offset != 0)
-      warning("When signed is TRUE, offset will be ignored")
+      rlang::warn("When signed is TRUE, offset will be ignored")
      for (i in seq_along(col_names))
        new_data[, col_names[i]] <-
          ifelse(abs(new_data[[ col_names[i] ]]) < 1,
