@@ -15,7 +15,7 @@ recipe <- function(x, ...)
 #' @rdname recipe
 #' @export
 recipe.default <- function(x, ...)
-  stop("`x` should be a data frame, matrix, or tibble", call. = FALSE)
+  rlang::abort("`x` should be a data frame, matrix, or tibble")
 
 #' @rdname recipe
 #' @param vars A character string of column names corresponding to variables
@@ -154,11 +154,17 @@ recipe.data.frame <-
 
     if (!is.null(formula)) {
       if (!is.null(vars))
-        stop("This `vars` specification will be ignored when a formula is ",
-             "used", call. = FALSE)
+        rlang::abort(
+          paste0("This `vars` specification will be ignored ",
+             "when a formula is used"
+             )
+          )
       if (!is.null(roles))
-        stop("This `roles` specification will be ignored when a formula is ",
-             "used", call. = FALSE)
+        rlang::abort(
+          paste0("This `roles` specification will be ignored ",
+             "when a formula is used"
+             )
+          )
 
       obj <- recipe.formula(formula, x, ...)
       return(obj)
@@ -171,9 +177,9 @@ recipe.data.frame <-
       x <- as_tibble(x)
 
     if (any(table(vars) > 1))
-      stop("`vars` should have unique members", call. = FALSE)
+      rlang::abort("`vars` should have unique members")
     if (any(!(vars %in% colnames(x))))
-      stop("1+ elements of `vars` are not in `x`", call. = FALSE)
+      rlang::abort("1+ elements of `vars` are not in `x`")
 
     x <- x[, vars]
 
@@ -182,8 +188,10 @@ recipe.data.frame <-
     ## Check and add roles when available
     if (!is.null(roles)) {
       if (length(roles) != length(vars))
-        stop("The number of roles should be the same as the number of ",
-             "variables", call. = FALSE)
+        rlang::abort(
+          paste0("The number of roles should be the same as the number of ",
+             "variables")
+        )
       var_info$role <- roles
     } else
       var_info$role <- NA
@@ -236,8 +244,9 @@ recipe.matrix <- function(x, ...) {
 form2args <- function(formula, data, ...) {
   if (!is_formula(formula))
     formula <- as.formula(formula)
+
   ## check for in-line formulas
-  element_check(formula, allowed = NULL)
+  inline_check(formula)
 
   if (!is_tibble(data))
     data <- as_tibble(data)
@@ -265,6 +274,20 @@ form2args <- function(formula, data, ...) {
   list(x = data, vars = vars, roles = roles)
 }
 
+inline_check <- function(x) {
+  funs <- fun_calls(x)
+  funs <- funs[!(funs %in% c("~", "+", "-"))]
+
+  if (length(funs) > 0) {
+    rlang::abort(paste0(
+      "No in-line functions should be used here; ",
+      "use steps to define baking actions."
+    ))
+  }
+
+  invisible(x)
+}
+
 
 #' @aliases prep prep.recipe
 #' @param x an object
@@ -275,7 +298,7 @@ form2args <- function(formula, data, ...) {
 #' @concept preprocessing
 #' @concept model_specification
 #' @export
-prep   <- function(x, ...)
+prep <- function(x, ...)
   UseMethod("prep")
 
 #' Train a Data Recipe
@@ -290,6 +313,8 @@ prep   <- function(x, ...)
 #'   `training`.
 #' @param verbose A logical that controls whether progress is reported as operations
 #'   are executed.
+#' @param log_changes A logical for printing a summary for each step regarding
+#'  which (if any) columns were added or removed during training.
 #' @param retain A logical: should the *preprocessed* training set be saved
 #'   into the `template` slot of the recipe after training? This is a good
 #'     idea if you want to add more steps later but want to avoid re-training
@@ -321,6 +346,27 @@ prep   <- function(x, ...)
 #'   if the first step is to center the data and the second is to scale the
 #'   data, the step for scaling is given the centered data.
 #'
+#'
+#' @examples
+#' data(ames, package = "modeldata")
+#'
+#' library(dplyr)
+#'
+#' ames <- mutate(ames, Sale_Price = log10(Sale_Price))
+#'
+#' ames_rec <-
+#'   recipe(
+#'     Sale_Price ~ Longitude + Latitude + Neighborhood + Year_Built + Central_Air,
+#'     data = ames
+#'   ) %>%
+#'   step_other(Neighborhood, threshold = 0.05) %>%
+#'   step_dummy(all_nominal()) %>%
+#'   step_interact(~ starts_with("Central_Air"):Year_Built) %>%
+#'   step_ns(Longitude, Latitude, deg_free = 5)
+#'
+#' prep(ames_rec, verbose = TRUE)
+#'
+#' prep(ames_rec, log_changes = TRUE)
 #' @rdname prep
 #' @export
 prep.recipe <-
@@ -329,46 +375,11 @@ prep.recipe <-
            fresh = FALSE,
            verbose = FALSE,
            retain = TRUE,
+           log_changes = FALSE,
            strings_as_factors = TRUE,
            ...) {
 
-    # In case a variable has multiple roles
-    vars <- unique(x$var_info$variable)
-
-    if (is.null(training)) {
-      if (fresh)
-        stop("A training set must be supplied to the `training` argument ",
-             "when `fresh = TRUE`", call. = FALSE)
-      training <- x$template
-    } else {
-      if (!all(vars %in% colnames(training))) {
-        stop("Not all variables in the recipe are present in the supplied ",
-             "training set", call. = FALSE)
-      }
-      training <- if (!is_tibble(training))
-        as_tibble(training[, vars, drop = FALSE])
-      else
-        training[, vars]
-    }
-
-    steps_trained <- vapply(x$steps, is_trained, logical(1))
-    if (any(steps_trained) & !fresh) {
-      if(!x$retained)
-        stop(
-          "To prep new steps after prepping the original ",
-          "recipe, `retain = TRUE` must be set each time that ",
-          "the recipe is trained.",
-          call. = FALSE
-        )
-      if (!is.null(x$training))
-        warning(
-          "The previous data will be used by `prep`; ",
-          "the data passed using `training` will be ",
-          "ignored.",
-          call. = FALSE
-        )
-      training <- x$template
-    }
+    training <- check_training_set(training, x, fresh)
 
     tr_data <- train_info(training)
 
@@ -386,21 +397,40 @@ prep.recipe <-
     # use `retain = TRUE` so issue a warning if this is not the case
     skippers <- map_lgl(x$steps, is_skipable)
     if (any(skippers) & !retain)
-      warning("Since some operations have `skip = FALSE`, using ",
-              "`retain = TRUE` will allow those steps results to ",
-              "be accessible.")
+      rlang::warn(
+        paste0(
+          "Since some operations have `skip = TRUE`, using ",
+          "`retain = TRUE` will allow those steps results to ",
+          "be accessible."
+        )
+      )
 
 
     running_info <- x$term_info %>% mutate(number = 0, skip = FALSE)
-    for (i in seq(along = x$steps)) {
-      note <-
-        paste("oper",  i, gsub("_", " ", class(x$steps[[i]])[1]))
+    for (i in seq(along.with = x$steps)) {
+      needs_tuning <- map_lgl(x$steps[[i]], is_tune)
+      if (any(needs_tuning)) {
+        arg <- names(needs_tuning)[needs_tuning]
+        arg <- paste0("'", arg, "'", collapse = ", ")
+        msg <-
+          paste0(
+            "You cannot `prep()` a tuneable recipe. Argument(s) with `tune()`: ",
+            arg,
+            ". Do you want to use a tuning function such as `tune_grid()`?"
+          )
+        rlang::abort(msg)
+      }
+      note <- paste("oper",  i, gsub("_", " ", class(x$steps[[i]])[1]))
       if (!x$steps[[i]]$trained | fresh) {
-        if (verbose)
+
+        if (verbose) {
           cat(note, "[training]", "\n")
+        }
+
+        before_nms <- names(training)
+
         # Compute anything needed for the preprocessing steps
         # then apply it to the current training set
-
         x$steps[[i]] <-
           prep(x$steps[[i]],
                training = training,
@@ -411,7 +441,6 @@ prep.recipe <-
 
         # Update the roles and the term source
         if (!is.na(x$steps[[i]]$role)) {
-
           new_vars <- setdiff(x$term_info$variable, running_info$variable)
           pos_new_var <- x$term_info$variable %in% new_vars
           pos_new_and_na_role <- pos_new_var & is.na(x$term_info$role)
@@ -422,6 +451,8 @@ prep.recipe <-
 
         }
 
+        changelog(log_changes, before_nms, names(training), x$steps[[i]])
+
         running_info <- rbind(
           running_info,
           mutate(x$term_info, number = i, skip = x$steps[[i]]$skip)
@@ -479,12 +510,13 @@ bake <- function(object, ...)
 
 #' Apply a Trained Data Recipe
 #'
-#' For a recipe with at least one preprocessing operations that has been trained by
+#' For a recipe with at least one preprocessing operation that has been trained by
 #'   [prep.recipe()], apply the computations to new data.
 #' @param object A trained object such as a [recipe()] with at least
 #'   one preprocessing operation.
 #' @param new_data A data frame or tibble for whom the preprocessing will be
-#'   applied.
+#'   applied. If `NULL` is given to `new_data`, the pre-processed _training
+#'   data_ will be returned (assuming that `prep(retain = TRUE)` was used).
 #' @param ... One or more selector functions to choose which variables will be
 #'   returned by the function. See [selections()] for more details.
 #'   If no selectors are given, the default is to use
@@ -500,29 +532,55 @@ bake <- function(object, ...)
 #' @details [bake()] takes a trained recipe and applies the
 #'   operations to a data set to create a design matrix.
 #'
-#' If the original data used to train the data are to be
-#'  processed, time can be saved by using the `retain = TRUE` option
-#'  of [prep()] to avoid duplicating the same operations. With this
-#'  option set, [juice()] can be used instead of `bake` with
-#'  `new_data` equal to the training set.
+#' If the data set is not too large, time can be saved by using the
+#'  `retain = TRUE` option of [prep()]. This stores the processed version of the
+#'  training set. With this option set, `bake(object, new_data = NULL)`
+#'  will return it for free.
 #'
 #' Also, any steps with `skip = TRUE` will not be applied to the
-#'   data when `bake` is invoked. [juice()] will always have all
-#'   of the steps applied.
-#' @seealso [recipe()], [juice()], [prep()]
+#'   data when `bake()` is invoked with a data set in `new_data`.
+#'   `bake(object, new_data = NULL)` will always have all of the steps applied.
+#' @seealso [recipe()], [prep()]
 #' @rdname bake
+#' @examples
+#' data(ames, package = "modeldata")
+#'
+#' ames <- mutate(ames, Sale_Price = log10(Sale_Price))
+#'
+#' ames_rec <-
+#'   recipe(Sale_Price ~ ., data = ames[-(1:6), ]) %>%
+#'   step_other(Neighborhood, threshold = 0.05) %>%
+#'   step_dummy(all_nominal()) %>%
+#'   step_interact(~ starts_with("Central_Air"):Year_Built) %>%
+#'   step_ns(Longitude, Latitude, deg_free = 2) %>%
+#'   step_zv(all_predictors()) %>%
+#'   prep()
+#'
+#' # return the training set (already embedded in ames_rec)
+#' ames_train <- bake(ames_rec, new_data = NULL)
+#'
+#' # apply processing to other data:
+#' ames_new <- bake(ames_rec, new_data = head(ames))
 #' @export
-bake.recipe <- function(object, new_data = NULL, ..., composition = "tibble") {
+bake.recipe <- function(object, new_data, ..., composition = "tibble") {
+  if (rlang::is_missing(new_data)) {
+    rlang::abort("'new_data' must be either a data frame or NULL. No value is not allowed.")
+  }
+  if (is.null(new_data)) {
+    return(juice(object, ..., composition = composition))
+  }
+
   if (!fully_trained(object)) {
-    stop("At least one step has not been trained. Please ",
-         "run `prep`.",
-         call. = FALSE)
+    rlang::abort("At least one step has not been trained. Please run `prep`.")
   }
 
   if (!any(composition == formats)) {
-    stop("`composition` should be one of: ",
-         paste0("'", formats, "'", collapse = ","),
-         call. = FALSE)
+    rlang::abort(
+      paste0(
+      "`composition` should be one of: ",
+      paste0("'", formats, "'", collapse = ",")
+      )
+    )
   }
 
   terms <- quos(...)
@@ -533,10 +591,9 @@ bake.recipe <- function(object, new_data = NULL, ..., composition = "tibble") {
   # In case someone used the deprecated `newdata`:
   if (is.null(new_data) || is.null(ncol(new_data))) {
     if (any(names(terms) == "newdata")) {
-      stop("Please use `new_data` instead of `newdata` with `bake`.",
-           call. = FALSE)
+      rlang::abort("Please use `new_data` instead of `newdata` with `bake`.")
     } else {
-      stop("Please pass a data set to `new_data`.", call. = FALSE)
+      rlang::abort("Please pass a data set to `new_data`.")
     }
   }
 
@@ -546,51 +603,50 @@ bake.recipe <- function(object, new_data = NULL, ..., composition = "tibble") {
 
   check_nominal_type(new_data, object$orig_lvls)
 
-  # Determine return variables. The context (ie. `info`) can
-  # change depending on whether a skip step was used. If so, we
-  # use an alternate info tibble that has all possible terms
-  # in it.
-  has_skip <- vapply(object$steps, function(x) x$skip, logical(1))
+  # Drop completely new columns from `new_data` and reorder columns that do
+  # still exist to match the ordering used when training
+  original_names <- names(new_data)
+  original_training_names <- unique(object$var_info$variable)
+  bakeable_names <- intersect(original_training_names, original_names)
+  new_data <- new_data[, bakeable_names]
 
-  if (any(has_skip)) {
-    keepers <-
-      terms_select(terms = terms,
-                   info = object$last_term_info,
-                   empty_fun = passover)
-  } else {
-    keepers <-
-      terms_select(terms = terms,
-                   info = object$term_info,
-                   empty_fun = passover)
+  n_steps <- length(object$steps)
+
+  for (i in seq_len(n_steps)) {
+    step <- object$steps[[i]]
+
+    if (is_skipable(step)) {
+      next
+    }
+
+    new_data <- bake(step, new_data = new_data)
+
+    if (!is_tibble(new_data)) {
+      new_data <- as_tibble(new_data)
+    }
   }
 
-  if (length(keepers) > 0) {
-    for (i in seq(along = object$steps)) {
-      if (!is_skipable(object$steps[[i]])) {
-        new_data <- bake(object$steps[[i]], new_data = new_data)
-        if (!is_tibble(new_data))
-          new_data <- as_tibble(new_data)
-      }
-    }
+  # Use `last_term_info`, which maintains info on all columns that got added
+  # and removed from the training data. This is important for skipped steps
+  # which might have resulted in columns not being added/removed in the test
+  # set.
+  info <- object$last_term_info
 
-    new_data <- new_data[, names(new_data) %in% keepers]
-    columns_sorted <- match(keepers, names(new_data))
-    columns_sorted <- columns_sorted[!is.na(columns_sorted)]
-    new_data <- new_data[, columns_sorted]
-    ## The levels are not null when no nominal data are present or
-    ## if strings_as_factors = FALSE in `prep`
-    if (!is.null(object$levels)) {
-      var_levels <- object$levels
-      var_levels <- var_levels[keepers]
-      check_values <-
-        vapply(var_levels, function(x)
-          (!all(is.na(x))), c(all = TRUE))
-      var_levels <- var_levels[check_values]
-      if (length(var_levels) > 0)
-        new_data <- strings2factors(new_data, var_levels)
-    }
-  } else {
-    new_data <- tibble(.rows = nrow(new_data))
+  # Now reduce to only user selected columns
+  out_names <- eval_select_recipes(terms, new_data, info)
+  new_data <- new_data[, out_names]
+
+  ## The levels are not null when no nominal data are present or
+  ## if strings_as_factors = FALSE in `prep`
+  if (!is.null(object$levels)) {
+    var_levels <- object$levels
+    var_levels <- var_levels[out_names]
+    check_values <-
+      vapply(var_levels, function(x)
+        (!all(is.na(x))), c(all = TRUE))
+    var_levels <- var_levels[check_values]
+    if (length(var_levels) > 0)
+      new_data <- strings2factors(new_data, var_levels)
   }
 
   if (composition == "dgCMatrix") {
@@ -675,7 +731,7 @@ print.recipe <- function(x, form_width = 30, ...) {
 #' @examples
 #' rec <- recipe( ~ ., data = USArrests)
 #' summary(rec)
-#' rec <- step_pca(rec, all_numeric(), num = 3)
+#' rec <- step_pca(rec, all_numeric(), num_comp = 3)
 #' summary(rec) # still the same since not yet trained
 #' rec <- prep(rec, training = USArrests)
 #' summary(rec)
@@ -691,86 +747,64 @@ summary.recipe <- function(object, original = FALSE, ...) {
 
 #' Extract Finalized Training Set
 #'
+#' As of `recipes` version 0.1.14, **`juice()` is superseded** in favor of
+#' `bake(object, new_data = NULL)`.
+#'
 #' As steps are estimated by `prep`, these operations are
-#'  applied to the training set. Rather than running `bake`
+#'  applied to the training set. Rather than running `bake()`
 #'  to duplicate this processing, this function will return
 #'  variables from the processed training set.
 #' @inheritParams bake.recipe
 #' @param object A `recipe` object that has been prepared
 #'   with the option `retain = TRUE`.
 #' @details When preparing a recipe, if the training data set is
-#'  retained using `retain = TRUE`, there is no need to `bake` the
+#'  retained using `retain = TRUE`, there is no need to `bake()` the
 #'  recipe to get the preprocessed training set.
 #'
-#'  `juice` will return the results of a recipes where _all steps_
+#'  `juice()` will return the results of a recipe where _all steps_
 #'  have been applied to the data, irrespective of the value of
 #'  the step's `skip` argument.
-#'
-#' @examples
-#' library(modeldata)
-#' data(biomass)
-#'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
-#'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-#'               data = biomass_tr)
-#'
-#' sp_signed <- rec %>%
-#'   step_normalize(all_predictors()) %>%
-#'   step_spatialsign(all_predictors())
-#'
-#' sp_signed_trained <- prep(sp_signed, training = biomass_tr)
-#'
-#' tr_values <- bake(sp_signed_trained, new_data = biomass_tr, all_predictors())
-#' og_values <- juice(sp_signed_trained, all_predictors())
-#'
-#' all.equal(tr_values, og_values)
 #' @export
 #' @seealso [recipe()] [prep.recipe()] [bake.recipe()]
 juice <- function(object, ..., composition = "tibble") {
   if (!fully_trained(object)) {
-    stop("At least one step has not been trained. Please ",
-         "run `prep`.",
-         call. = FALSE)
+    rlang::abort("At least one step has not been trained. Please run `prep()`.")
   }
 
   if (!isTRUE(object$retained)) {
-    stop("Use `retain = TRUE` in `prep` to be able to extract the training set",
-         call. = FALSE)
+    rlang::abort(paste0(
+      "Use `retain = TRUE` in `prep()` to be able ",
+      "to extract the training set"
+    ))
   }
 
   if (!any(composition == formats)) {
-    stop("`composition` should be one of: ",
-         paste0("'", formats, "'", collapse = ","),
-         call. = FALSE)
+    rlang::abort(paste0(
+      "`composition` should be one of: ",
+      paste0("'", formats, "'", collapse = ",")
+    ))
   }
 
   terms <- quos(...)
   if (is_empty(terms)) {
     terms <- quos(everything())
   }
-  keepers <-
-    terms_select(terms = terms,
-                 info = object$term_info,
-                 empty_fun = passover)
 
-  if (length(keepers) > 0) {
-    new_data <- object$template[, names(object$template) %in% keepers]
+  # Get user requested columns
+  new_data <- object$template
+  out_names <- eval_select_recipes(terms, new_data, object$term_info)
+  new_data <- new_data[, out_names]
 
-    ## Since most models require factors, do the conversion from character
-    if (!is.null(object$levels)) {
-      var_levels <- object$levels
-      var_levels <- var_levels[keepers]
-      check_values <-
-        vapply(var_levels, function(x)
-          (!all(is.na(x))), c(all = TRUE))
-      var_levels <- var_levels[check_values]
-      if (length(var_levels) > 0)
-        new_data <- strings2factors(new_data, var_levels)
-    }
-  } else {
-    new_data <- tibble(.rows = nrow(object$template))
+  ## Since most models require factors, do the conversion from character
+  if (!is.null(object$levels)) {
+    var_levels <- object$levels
+    var_levels <- var_levels[out_names]
+    check_values <-
+      vapply(var_levels, function(x)
+        (!all(is.na(x))), c(all = TRUE))
+    var_levels <- var_levels[check_values]
+    if (length(var_levels) > 0)
+      new_data <- strings2factors(new_data, var_levels)
   }
 
   if (composition == "dgCMatrix") {
@@ -787,3 +821,24 @@ juice <- function(object, ..., composition = "tibble") {
 formats <- c("tibble", "dgCMatrix", "matrix", "data.frame")
 
 utils::globalVariables(c("number"))
+
+# ------------------------------------------------------------------------------
+
+#' @rdname required_pkgs
+#' @export
+required_pkgs.recipe <- function(x, infra = TRUE, ...) {
+  res <- purrr::map(x$steps, required_pkgs)
+  res <- unique(unlist(res))
+  if (infra) {
+    res <- c("recipes", res)
+  }
+  res <- unique(res)
+  res <- res[length(res) != 0]
+  res
+}
+
+#' @rdname required_pkgs
+#' @export
+required_pkgs.step <- function(x, ...) {
+  character(0)
+}
