@@ -6,7 +6,7 @@
 #' @inheritParams step_center
 #' @inherit step_center return
 #' @param ... One or more selector functions to choose which
-#'  variables that will evaluated by the filtering. See
+#'  variables that will be evaluated by the filtering. See
 #'  [selections()] for more details. For the `tidy`
 #'  method, these are not currently used.
 #' @param role Not used by this step since no new variables are
@@ -84,12 +84,14 @@ step_zv_new <-
     )
   }
 
-one_unique <- function(x)
-  length(unique(x)) == 1
+one_unique <- function(x) {
+  x <- x[!is.na(x)]
+  length(unique(x)) < 2
+  }
 
 #' @export
 prep.step_zv <- function(x, training, info = NULL, ...) {
-  col_names <- terms_select(x$terms, info = info)
+  col_names <- eval_select_recipes(x$terms, training, info)
   filter <- vapply(training[, col_names], one_unique, logical(1))
 
   step_zv_new(
