@@ -11,7 +11,10 @@ test_that("step_naomit on all columns", {
     prep(airquality, verbose = FALSE) %>%
     juice()
 
-  expect_equal(baked, na.omit(airquality))
+  na_res <- tibble(na.omit(airquality))
+  attributes(na_res)$na.action <- NULL
+
+  expect_equal(baked, na_res[, c(2:6, 1)])
 })
 
 test_that("step_naomit on subset of columns", {
@@ -21,14 +24,18 @@ test_that("step_naomit on subset of columns", {
     prep(airquality, verbose = FALSE) %>%
     juice()
 
-  expect_equal(baked, tidyr::drop_na(airquality, Ozone, Solar.R))
+  na_res <- tibble(tidyr::drop_na(airquality, Ozone, Solar.R))
+
+  expect_equal(baked, na_res[, c(2:6, 1)])
 
   baked2 <- recipe(Ozone ~ ., data = airquality) %>%
     step_naomit(Solar.R) %>%
     prep(airquality, verbose = FALSE) %>%
     juice()
 
-  expect_equal(baked2, tidyr::drop_na(airquality, Solar.R))
+  na_res2 <- tibble(tidyr::drop_na(airquality, Solar.R))
+
+  expect_equal(baked2, na_res2[, c(2:6, 1)])
 })
 
 test_that("something prints", {
